import os.path
import subprocess
import time


class Monitor:
    def __init__(self):
        self.status = {
            "mysql": True,
            "nginx": True,
            "memcached": True,
            "php-fpm-74": True,
        }
        self.check = {
            "last_check": 0.0,
            "failed_time": 0.0,
            "failed_send_time": 0.0,
            "success_send_time": 0.0
        }

    def mysql(self):
        process = subprocess.run(["pgrep", "-x", "mysqld"], capture_output=True)
        if process.returncode != 0:
            self.status.update({"mysql": False})
            process = subprocess.run(['/etc/init.d/mysqld', 'start'])
            print(process.stdout)
        else:
            self.status.update({"mysql": True})

    def nginx(self):
        nginx_shell = '/etc/init.d/nginx'
        process = subprocess.run([nginx_shell, "status"], capture_output=True)
        if process.stdout.decode("utf8").find("running") == -1:
            self.status.update({"nginx": False})
            # process = subprocess.run([nginx_shell, 'start'])
            # print(process.stdout)
        else:
            self.status.update({"nginx": True})

    def memcached(self):
        process = subprocess.run(["pgrep", "-x", "memcached"], capture_output=True)
        if process.returncode != 0:
            self.status.update({"memcached": False})
            process = subprocess.run(['/etc/init.d/memcached', 'start'])
            print(process.stdout)
        else:
            self.status.update({"memcached": True})

    def php_fpm_74(self):
        php_74 = "/etc/init.d/php-fpm-74"
        if os.path.exists(php_74):
            process = subprocess.run([php_74, "status"], capture_output=True)
            if process.stdout.decode("utf8").find("running") == -1:
                self.status.update({"php-fpm-74": False})
                process = subprocess.run([php_74, 'start'])
                print(process.stdout)
            else:
                self.status.update({"php-fpm-74": True})

    def is_failed(self):
        failed = dict((k, v) for (k, v) in self.status.items() if not v)
        return len(failed.items()) > 0

    def run(self):
        self.mysql()
        self.memcached()
        self.php_fpm_74()
        self.nginx()
        cache = os.path.join(os.getcwd(), "cache")
        if not os.path.exists(cache):
            os.makedirs(cache)

        self.check.update({"last_check": time.time()})

        if self.is_failed() and not self.check.get("failed_time"):
            self.check.update({"failed_time": time.time()})


if __name__ == "__main__":
    monitor = Monitor()
    monitor.run()



from .monitor import Monitor
from .ding import Ding
import time
from dotenv import load_dotenv

load_dotenv()

monitor = Monitor()


def run():
    while True:
        monitor.run()
        print(monitor.is_failed(), monitor.check.get("failed_time"))
        if monitor.is_failed() and monitor.check.get("failed_time") \
                and ((time.time() - monitor.check.get("failed_send_time")) > 20 or
                     not monitor.check.get("failed_send_time")):
            ding = Ding()
            monitor.check.update({"failed_send_time": time.time()})
            print(monitor.status)
            msg = [k + " 服务停止了." for (k, v) in monitor.status.items() if not v]
            if ding.send_text(" > ".join(msg)):
                monitor.failed_send_time = time.time()
        elif not monitor.is_failed() and monitor.check.get("failed_time"):
            monitor.check.update({"failed_time": 0.0})
            print("success")

        # send
        time.sleep(3)



import os
import time
import hmac
import hashlib
import base64
import urllib.parse
import requests


class Ding:
    def __init__(self, token="", sign=""):
        self.token = token
        if token == "":
            self.token = os.getenv("DING_TOKEN")
        self.sign = sign
        if sign == "":
            self.sign = os.getenv("DING_SIGN")

    def _sign(self):
        timestamp = str(round(time.time() * 1000))
        secret = self.sign
        secret_enc = secret.encode('utf-8')
        string_to_sign = '{}\n{}'.format(timestamp, secret)
        string_to_sign_enc = string_to_sign.encode('utf-8')
        hmac_code = hmac.new(secret_enc, string_to_sign_enc, digestmod=hashlib.sha256).digest()
        sign = urllib.parse.quote_plus(base64.b64encode(hmac_code))
        return f'{self.token}&timestamp={timestamp}&sign={sign}'

    #
    def send_text(self, message):
        url = self._sign()
        data = {
            'msgtype': 'markdown',
            "markdown": {
                "title": "Server notification",
                "text": message
            }
        }
        response = requests.post(url, json=data, headers={"Content-Type": "application/json;charset=utf-8"})
        return response.status_code == 200


if __name__ == "__main__":
    ding = Ding()
    ding.send_text("test.")
